@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -479,15 +479,19 @@ class Account(base.Base, authenticable.Authenticable):
     @classmethod
     @appier.operation(
         name = "Create",
+        description = """Create a new account with the provided
+        username, email and password, in case no password is
+        provided a random new one is generated""",
         parameters = (
             ("Username", "username", str),
             ("Email", "email", str),
-            ("Password", "password", str),
+            ("Password", "password", str, ""),
             ("Send Email", "send_email", bool, False)
         ),
         factory = True
     )
-    def create_s(cls, username, email, password, send_email):
+    def create_s(cls, username, email, password = "", send_email = False):
+        if not password: password = appier.gen_token(limit = 16)
         account = cls(
             username = username,
             email = email,
@@ -498,6 +502,46 @@ class Account(base.Base, authenticable.Authenticable):
         if send_email: account.email_new(password = password)
         return account
 
+    @classmethod
+    @appier.operation(
+        name = "Import JSON",
+        parameters = (
+            ("JSON File", "file", "file"),
+            ("Empty source", "empty", bool, False)
+        )
+    )
+    def import_json_s(cls, file, empty):
+
+        def callback(item):
+
+            enabled = item["enabled"]
+            username = item["username"]
+            type = item["type"]
+            password = item.get("password", None)
+            email = item.get("email", None)
+            key = item.get("key", None)
+            description = item.get("description", None)
+            last_login = item.get("last_login", None)
+            avatar = item.get("avatar", None)
+            roles = item.get("roles", [])
+            account = cls(
+                enabled = enabled,
+                username = username,
+                type = type,
+                password = password,
+                password_confirm = password,
+                email = email,
+                key = key,
+                description = description,
+                last_login = last_login,
+                avatar = avatar,
+                roles = roles
+            )
+            account.save()
+
+        if empty: cls.delete_c()
+        cls._json_import(file, callback)
+
     @classmethod
     @appier.operation(
         name = "Import CSV",
@@ -506,7 +550,7 @@ class Account(base.Base, authenticable.Authenticable):
             ("Empty source", "empty", bool, False)
         )
     )
-    def import_csv_s(cls, file, empty):
+    def import_csv_s(cls, file, empty = False):
 
         def callback(line):
             username,\
@@ -596,8 +640,11 @@ class Account(base.Base, authenticable.Authenticable):
 
     def pre_create(self):
         base.Base.pre_create(self)
-        self.key = self.secret()
-        self.confirmation_token = self.secret()
+        if not hasattr(self, "key") or not self.key:
+            self.key = self.secret()
+        if not hasattr(self, "confirmation_token") or\
+            not self.confirmation_token:
+            self.confirmation_token = self.secret()
         if not hasattr(self, "avatar") or not self.avatar:
             self._set_avatar_d()
 
@@ -629,18 +676,6 @@ class Account(base.Base, authenticable.Authenticable):
         self.reset_token = None
         self.save()
 
-    def add_role_s(self, name):
-        _role = role.Role.get(name = name)
-        if _role in self.roles_l: return
-        self.roles_l.append(_role)
-        self.save()
-
-    def remove_role_s(self, name):
-        _role = role.Role.get(name = name)
-        if not _role in self.roles_l: return
-        self.roles_l.remove(_role)
-        self.save()
-
     def tokens(self):
         """
         Retrieves the complete set of ACL tokens for the current use
@@ -664,7 +699,7 @@ class Account(base.Base, authenticable.Authenticable):
             tokens.update(["base", "user"])
 
         for role in self.roles_l:
-            tokens.update(role.tokens)
+            tokens.update(role.tokens_a)
 
         if "*" in tokens: tokens = ["*"]
 
@@ -693,6 +728,37 @@ class Account(base.Base, authenticable.Authenticable):
             views.append(view_m)
         return views
 
+    def meta_m(self, join = False):
+        """
+        Merges the metadata dictionary of the current account to the ones
+        defined in the complete set of associated roles.
+
+        :type join: bool
+        :param join: If the overlapping values between the operations
+        should be merged as sequences.
+        :rtype: Dictionary
+        :return: The final meta-data dictionary that contains the "merged"
+        view of the metadata for the account according to the associated
+        set of roles.
+        """
+
+        meta = dict(self.meta)
+        for role in self.roles_l:
+            if join:
+                for key, value in appier.legacy.iteritems(role.meta_a):
+                    if key in meta:
+                        previous = meta[key]
+                        if not type(previous) == list:
+                            previous = [previous]
+                        if not value in previous:
+                            value = previous + [value]
+                        else:
+                            value = previous
+                    meta[key] = value
+            else:
+                meta.update(role.meta_a)
+        return meta
+
     def type_s(self, capitalize = False):
         type_s = Account.ACCOUNT_S.get(self.type, None)
         type_s = type_s.capitalize() if capitalize else type_s
@@ -823,6 +889,26 @@ class Account(base.Base, authenticable.Authenticable):
         self.avatar = cls.avatar.type(avatar)
         self.save()
 
+    @appier.operation(
+        name = "Add Role",
+        parameters = (("Name", "name", str),)
+    )
+    def add_role_s(self, name):
+        _role = role.Role.get(name = name)
+        if _role in self.roles_l: return
+        self.roles_l.append(_role)
+        self.save()
+
+    @appier.operation(
+        name = "Remove Role",
+        parameters = (("Name", "name", str),)
+    )
+    def remove_role_s(self, name):
+        _role = role.Role.get(name = name)
+        if not _role in self.roles_l: return
+        self.roles_l.remove(_role)
+        self.save()
+
     @appier.link(name = "View Avatar", devel = True)
     def view_avatar_url(self, absolute = False):
         cls = self.__class__
@@ -834,6 +920,16 @@ class Account(base.Base, authenticable.Authenticable):
             absolute = absolute
         )
 
+    @classmethod
+    @appier.link(name = "Export JSON", context = True)
+    def export_url(cls, view = None, context = None, absolute = False):
+        return appier.get_app().url_for(
+            "admin.export_accounts_json",
+            view = view,
+            context = context,
+            absolute = absolute
+        )
+
     @property
     def confirmed(self):
         return self.enabled
