@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -42,6 +42,7 @@ import json
 import time
 import random
 import string
+import inspect
 import hashlib
 import logging
 import datetime
@@ -325,8 +326,13 @@ class Base(appier.Model):
             if appier.legacy.is_str(view):
                 view_cls = owner.get_model(view)
                 view = view_cls.view_r
+            is_class = inspect.isclass(view)
+            if is_class:
+                view = view.view_r
             is_callable = hasattr(view, "__call__")
-            if is_callable: view = view(target = cls, owner = owner)
+            if is_callable:
+                view = view(target = cls, owner = owner)
+            if not view: continue
             object.update(view)
 
         # returns the final object to the caller method so that it can be
@@ -346,6 +352,7 @@ class Base(appier.Model):
         callback_header = None,
         strict = False,
         named = False,
+        header = True,
         delimiter = ",",
         quotechar = "\"",
         quoting = csv.QUOTE_MINIMAL,
@@ -364,7 +371,8 @@ class Base(appier.Model):
         args, _varargs, kwargs = appier.legacy.getargspec(callback)[:3]
         has_header = True if "header" in args or kwargs else False
         has_map = True if "map" in args or kwargs else False
-        header = next(csv_reader)
+        if header: header = next(csv_reader)
+        else: header = []
         if not is_unicode: header = [value.decode(encoding) for value in header]
         if callback_header: callback_header(header)
         if named: tuple_t = collections.namedtuple("csv_tuple", header)
@@ -460,7 +468,7 @@ class Base(appier.Model):
             strip_important = strip_important,
             cssutils_logging_level = logging_level
         )
-        return inliner.transform(data)
+        return inliner.transform()
 
     @classmethod
     def _inlinify_toronado(cls, data, *args, **kwargs):
@@ -480,7 +488,7 @@ class Base(appier.Model):
         appier.Model.pre_create(self)
 
         if not hasattr(self, "enabled"): self.enabled = True
-        self.created = int(time.time())
+        if not hasattr(self, "created"): self.created = int(time.time())
         self.modified = int(time.time())
 
     def pre_update(self):
@@ -665,6 +673,16 @@ class Base(appier.Model):
         self.modified = int(self.modified)
         self.save(immutables_a = False)
 
+    @appier.operation(
+        name = "Set Metadata",
+        description = """Sets the "extra" metadata information on the current
+        entity, such value should be compliant with the JSON specification""",
+        parameters = (("Metadata", "meta", "longmap"),)
+    )
+    def set_metadata_s(self, meta):
+        self.meta = meta
+        self.save()
+
     @property
     def created_d(self):
         return datetime.datetime.fromtimestamp(self.created)
