@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -41,6 +41,7 @@ import json
 
 import appier
 
+from appier_extras import utils
 from appier_extras.parts.admin.models import base
 
 class Event(base.Base):
@@ -102,11 +103,12 @@ class Event(base.Base):
 
     @classmethod
     def format(cls, arguments, all = False):
+        formatter = utils.SafeFormatter()
         arguments = dict(arguments)
         for key, value in appier.legacy.items(arguments):
             if not appier.legacy.is_string(value, all = all): continue
-            try: value = value.format(**arguments)
-            except: value = value
+            try: value = formatter.format(value, **arguments)
+            except Exception: value = value
             arguments[key] = value
         return arguments
 
@@ -177,7 +179,7 @@ class Event(base.Base):
         while True:
             try:
                 result = callable()
-            except:
+            except Exception:
                 count -= 1
                 if count == 0: raise
                 else: continue
