@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -48,6 +48,10 @@ class Locale(base.Base):
         default = True
     )
 
+    context = appier.field(
+        index = "all"
+    )
+
     data_j = appier.field(
         type = dict,
         private = True,
@@ -72,8 +76,7 @@ class Locale(base.Base):
         return super(Locale, cls).validate() + [
             appier.not_null("locale"),
             appier.not_empty("locale"),
-            appier.is_regex("locale", "^[a-z]{2}(?:_[a-z]{2})?$"),
-            appier.not_duplicate("locale", cls._name()),
+            appier.is_regex("locale", "^[a-z]{2}(?:_[a-z]{2}(?:_[a-z]+)?)?$"),
 
             appier.not_null("data_j"),
             appier.not_empty("data_j")
@@ -81,27 +84,30 @@ class Locale(base.Base):
 
     @classmethod
     def list_names(cls):
-        return ["locale", "description", "count_l"]
+        return ["locale", "context", "description", "count_l"]
 
     @classmethod
     def bundles_d(cls, locale = None):
         locales = cls.find_e(rules = False)
-        return dict([(locale.locale, locale.data_u) for locale in locales])
+        return dict([((locale.locale, locale.context or None), locale.data_u) for locale in locales])
 
     @classmethod
     @appier.operation(
         name = "Import Bundle",
         parameters = (
             ("JSON File", "file", "file"),
-            ("Locale", "locale", str)
+            ("Locale", "locale", str),
+            ("Context", "context", str)
         ),
         factory = True
     )
-    def import_bundle_s(cls, file, locale, strict = False):
+    def import_bundle_s(cls, file, locale, context = None, strict = False):
+        context = context or None
         data_j = cls._json_read(file)
-        locale_e = cls.get(locale = locale, rules = False, raise_e = False)
+        conditions = cls._conditions(locale, context = context)
+        locale_e = cls.get(rules = False, raise_e = False, **{"$and" : conditions})
         if locale_e: locale_e.data_j.update(data_j)
-        else: locale_e = cls(locale = locale, data_j = data_j)
+        else: locale_e = cls(locale = locale, context = context, data_j = data_j)
         locale_e.save()
         return locale_e
 
@@ -124,7 +130,8 @@ class Locale(base.Base):
                 result[locale_n] = locale_m
 
         for locale, data_j in appier.legacy.iteritems(result):
-            locale_e = cls.get(locale = locale, rules = False, raise_e = False)
+            conditions = cls._conditions(locale)
+            locale_e = cls.get(rules = False, raise_e = False, **{"$and" : conditions})
             if locale_e: locale_e.data_j.update(data_j)
             else: locale_e = cls(locale = locale, data_j = data_j)
             locale_e.save()
@@ -137,8 +144,8 @@ class Locale(base.Base):
 
         # iterates over the complete set of locale data pairs
         # in the bundles dictionary to set these bundles (locales)
-        for locale, data_j in appier.legacy.iteritems(bundles_d):
-            appier.get_app()._register_bundle(data_j, locale)
+        for (locale, context), data_j in appier.legacy.iteritems(bundles_d):
+            appier.get_app()._register_bundle(data_j, locale, context = context)
 
     @classmethod
     def _escape(cls, data_j, target = ".", sequence = "::"):
@@ -154,6 +161,14 @@ class Locale(base.Base):
     def _unescape(cls, data_j, target = ".", sequence = "::"):
         return cls._escape(data_j, target = sequence, sequence = target)
 
+    @classmethod
+    def _conditions(cls, locale, context = None):
+        conditions = []
+        conditions.append(dict(locale = locale))
+        if context: conditions.append(dict(context = context))
+        else: conditions.append({"$or" : [dict(context = ""), dict(context = None)]})
+        return conditions
+
     def pre_create(self):
         base.Base.pre_create(self)
         self.count_l = len(self.data_j)
@@ -176,6 +191,14 @@ class Locale(base.Base):
         base.Base.post_delete(self)
         self.owner.trigger_bus("locale/reload")
 
+    @appier.link(name = "Export Bundle")
+    def bundle_url(self, absolute = False):
+        return appier.get_app().url_for(
+            "admin.bundle_locale_json",
+            id = self.id,
+            absolute = absolute
+        )
+
     @property
     def data_u(self):
         cls = self.__class__
