@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -85,6 +85,13 @@ class OAuthClient(base.Base):
     """ The redirect uri where to redirect, the user agent
     after a successful token request operation """
 
+    scope = appier.field(
+        type = list
+    )
+    """ The sequence of scope values allowed for the current
+    client, if not set all of the scope values are considered
+    to be allowed """
+
     @classmethod
     def validate(cls):
         return super(OAuthClient, cls).validate() + [
@@ -104,6 +111,23 @@ class OAuthClient(base.Base):
     def list_names(cls):
         return ["name", "client_id", "redirect_uri"]
 
+    @classmethod
+    @appier.operation(
+        name = "Create",
+        parameters = (
+            ("Name", "name", str),
+            ("Redirect URI", "redirect_uri", str, "http://example.com/oauth")
+        ),
+        factory = True
+    )
+    def create_s(cls, name, redirect_uri):
+        oauth_client = cls(
+            name = name,
+            redirect_uri = redirect_uri
+        )
+        oauth_client.save()
+        return oauth_client
+
     @classmethod
     def _underscore(cls, plural = True):
         return "oauth_clients" if plural else "oauth_client"
@@ -118,6 +142,24 @@ class OAuthClient(base.Base):
         self.client_id = appier.gen_token(hash = hashlib.md5)
         self.client_secret = appier.gen_token()
 
+    def assert_redirect_uri(self, redirect_uri):
+        appier.verify(
+            redirect_uri == self.redirect_uri,
+            message = "The provided redirect URI is not valid",
+            exception = appier.SecurityError,
+            code = 403
+        )
+
+    def assert_scope(self, scope):
+        if not self.scope: return
+        invalid = [token for token in scope if not token in self.scope]
+        appier.verify(
+            not invalid,
+            message = "The provided scope is not valid",
+            exception = appier.SecurityError,
+            code = 403
+        )
+
     def get_tokens(self):
         return oauth_token.OAuthToken.find(client = self.id)
 
