@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -62,6 +62,16 @@ class Role(base.Base):
     """ The filtered view that is going to be applied for
     every filtered operation (data source access) """
 
+    children = appier.field(
+        type = appier.references(
+            "Role",
+            name = "id"
+        )
+    )
+    """ The complete set of child roles that are associated
+    with this role, this role should inherit all og the characteristics
+    of the child roles (expected behaviour) """
+
     @classmethod
     def setup(cls):
         super(Role, cls).setup()
@@ -106,3 +116,66 @@ class Role(base.Base):
 
     def view_m(self, context = None):
         return self.view
+
+    @property
+    def tokens_a(self):
+        tokens = set(self.tokens)
+        for child in self.children: tokens.update(child.tokens_a)
+        return tokens
+
+    @property
+    def meta_a(self):
+        meta = dict(self.meta)
+        for child in self.children:
+            for key, value in appier.legacy.iteritems(child.meta):
+                if key in meta:
+                    previous = meta[key]
+                    if not type(previous) == list:
+                        previous = [previous]
+                    if not value in previous:
+                        value = previous + [value]
+                    else:
+                        value = previous
+                meta[key] = value
+        return meta
+
+    @appier.operation(
+        name = "Duplicate",
+        description = """Create a new account with exactly the
+        same specification as the current one""",
+        parameters = (("Suffix", "suffix", str, "-new"),),
+        factory = True
+    )
+    def duplicate_s(self, suffix = "-new"):
+        cls = self.__class__
+        role = cls(
+            description = self.description,
+            meta = self.meta,
+            name = self.name + suffix,
+            tokens = self.tokens,
+            view = self.view
+        )
+        role.save()
+        return role
+
+    @appier.operation(
+        name = "Set Parent",
+        parameters = (("Name", "name", str),)
+    )
+    def set_parent_s(self, name):
+        cls = self.__class__
+        parent = cls.get(name = name)
+        if self in parent.children: return
+        parent.children.append(self)
+        parent.save()
+
+    @appier.operation(
+        name = "Add Child",
+        parameters = (("Name", "name", str),)
+    )
+    def add_child_s(self, name):
+        cls = self.__class__
+        child = cls.get(name = name)
+        if child in self.children: return
+        self.children.append(child)
+        self.save()
