@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -50,6 +50,15 @@ from appier_extras import utils
 from appier_extras.parts.admin import models
 from appier_extras.parts.admin import social
 
+OAUTH_ERROR_CODES = {
+    400 : "invalid_request",
+    403 : "access_denied",
+    500 : "server_error"
+}
+""" Dictionary that maps the typical HTTP error codes into
+the string based OAuth error codes to be returned to the
+redirect URI on the authorize call (in case of error) """
+
 class AdminPart(
     appier.Part,
     social.Facebook,
@@ -103,6 +112,7 @@ class AdminPart(
         self._last_login = None
         self._login_count = 0
         self._sections = appier.OrderedDict()
+        self._operations = appier.OrderedDict()
 
     def version(self):
         return base.VERSION
@@ -121,7 +131,7 @@ class AdminPart(
         self.logger.debug("Registering pre request handler ...")
         appier.App.add_custom("before_request", self.before_request)
         appier.App.add_exception(
-            BaseException,
+            Exception,
             self.exception_handler,
             scope = AdminPart
         )
@@ -145,6 +155,10 @@ class AdminPart(
         self.owner.admin_oauth = self.oauth
         self.owner.admin_avatar_default = self.avatar_default
 
+        self.owner.admin_layout_fluid = "admin/layout.fluid.html.tpl"
+        self.owner.admin_layout_static = "admin/layout.static.html.tpl"
+        self.owner.admin_email_layout = "admin/email/layout.html.tpl"
+
         self.owner.lib_loaders["appier_extras"] = self._appier_extras_loader
         self.owner.lib_loaders["netius"] = self._netius_loader
         self.owner.lib_loaders["pconvert"] = self._pconvert_loader
@@ -170,6 +184,7 @@ class AdminPart(
             method()
 
         self.load_settings()
+        self.load_operations()
 
         self.account_c.bind_g("touch_login", self._on_touch_login)
 
@@ -177,6 +192,10 @@ class AdminPart(
         appier.Part.unload(self)
 
         self.account_c.unbind_g("touch_login", self._on_touch_login)
+
+        self.unload_operations()
+        self.unload_settings()
+
         self.owner.remove_filter("markdown")
 
     def routes(self):
@@ -207,6 +226,7 @@ class AdminPart(
             (("GET",), "/admin/oauth/deny", self.oauth_deny),
             (("GET", "POST"), "/admin/oauth/access_token", self.oauth_access_token, None, True),
             (("GET", "POST"), "/admin/oauth/login", self.oauth_login, None, True),
+            (("GET",), "/admin/operations/restart_app", self.restart_app),
             (("GET",), "/admin/operations/build_index", self.build_index),
             (("GET",), "/admin/operations/build_index_db", self.build_index_db),
             (("GET",), "/admin/operations/test_email", self.test_email),
@@ -217,8 +237,10 @@ class AdminPart(
             (("GET",), "/admin/sessions/me/delete", self.delete_session_me),
             (("GET",), "/admin/sessions/<str:sid>", self.show_session),
             (("GET",), "/admin/sessions/<str:sid>/delete", self.delete_session),
+            (("GET",), "/admin/peers", self.list_peers),
             (("GET",), "/admin/counters", self.list_counters),
             (("GET",), "/admin/events.csv", self.list_events_csv),
+            (("GET",), "/admin/locales/<int:id>/bundle.json", self.bundle_locale_json, None, True),
             (("GET",), "/admin/database", self.database),
             (("GET",), "/admin/database/export", self.database_export),
             (("GET",), "/admin/database/import", self.database_import),
@@ -228,6 +250,7 @@ class AdminPart(
             (("GET",), "/admin/accounts/new", self.new_account),
             (("POST",), "/admin/accounts", self.create_account),
             (("GET",), "/admin/accounts/me", self.me_account),
+            (("GET",), "/admin/accounts/export.json", self.export_accounts_json, None, True),
             (("GET",), "/admin/accounts/<str:username>", self.show_account),
             (("GET",), "/admin/accounts/<str:username>/mail", self.mail_account),
             (("GET",), "/admin/accounts/<str:username>/avatar", self.avatar_account),
@@ -280,9 +303,9 @@ class AdminPart(
         if not self.available: return None
         return models
 
-    def template(self, template, layout = None, *args, **kwargs):
-        layout = self.session.get("layout", self.layout)
-        template = "%s/%s" % (layout, template)
+    def template(self, template, layout = True, *args, **kwargs):
+        layout = self.session.get("layout", self.layout) if layout else None
+        template = "%s/%s" % (layout, template) if layout else template
         return appier.Part.template(
             self,
             template,
@@ -297,7 +320,7 @@ class AdminPart(
         if not key: return
         try: account = self.account_c.login_key(key)
         except appier.OperationalError: pass
-        else: account._set_session(method = "set_t")
+        else: account._set_account(method = "set_t")
 
     def exception_handler(self, error):
         import traceback
@@ -318,6 +341,12 @@ class AdminPart(
             value = settings.get(name, current)
             setattr(self, name, value)
 
+    def unload_settings(self):
+        settings = self.owner.get_preference(self.name() + ":settings")
+        if not settings: return
+        for name in self.settings_l:
+            delattr(self, name)
+
     def dump_settings(self):
         settings = dict()
         for name in self.settings_l:
@@ -328,6 +357,46 @@ class AdminPart(
     def flush_settings(self):
         self.dump_settings()
 
+    def load_operations(self):
+        self.add_operation(
+            "restart_app", "admin.restart_app",
+            description = "Restart application",
+            message = "Are you really sure you want to restart the application?",
+            note = "Restarting the application, may delete the system state",
+            level = 3
+        )
+        self.add_operation(
+            "build_index", "admin.build_index",
+            description = "Build search index",
+            message = "Are you really sure you want to rebuild the search index?",
+            note = "Rebuilding the complete search index, may take some time",
+            level = 3
+        )
+        self.add_operation(
+            "build_index_db", "admin.build_index_db",
+            description = "Build database index",
+            message = "Are you really sure you want to rebuild the database index?",
+            note = "Rebuilding the complete database index, may take some time",
+            level = 3
+        )
+        self.add_operation(
+            "test_email", "admin.test_email",
+            description = "Send test email",
+            note = "Sending this email is going to use loaded SMTP configuration"
+        )
+        self.add_operation(
+            "test_event", "admin.test_event",
+            description = "Trigger test event",
+            note = "All handlers for the event are going to be triggered"
+        )
+
+    def unload_operations(self):
+        self.remove_operation("restart_app")
+        self.remove_operation("build_index")
+        self.remove_operation("build_index_db")
+        self.remove_operation("test_email")
+        self.remove_operation("test_event")
+
     def add_section(self, name):
         self._sections[name] = appier.OrderedDict()
 
@@ -344,6 +413,33 @@ class AdminPart(
         if self._sections[section]: return
         del self._sections[section]
 
+    def add_operation(
+        self,
+        name,
+        route,
+        description = None,
+        message = None,
+        note = None,
+        parameters = (),
+        level = 1,
+        args = [],
+        kwargs = {}
+    ):
+        self._operations[name] = dict(
+            name = name,
+            route = route,
+            description = description,
+            message = message,
+            note = note,
+            parameters = parameters,
+            level = level,
+            args = args,
+            kwargs = kwargs
+        )
+
+    def remove_operation(self, name):
+        del self._operations[name]
+
     def index(self):
         return self.list_models()
 
@@ -384,7 +480,7 @@ class AdminPart(
 
         # updates the current session with the proper
         # values to correctly authenticate the user
-        account._set_session()
+        account._set_account()
 
         # redirects the current operation to the next URL or in
         # alternative to the root index of the administration
@@ -529,6 +625,21 @@ class AdminPart(
             account = account
         )
 
+    @appier.ensure(context = "admin")
+    def export_accounts_json(self):
+        account_c = self._get_cls(self.account_c)
+        object = appier.get_object(
+            alias = True,
+            find = True,
+            limit = 0
+        )
+        return self.admin_part._find_view(
+            account_c,
+            map = True,
+            rules = False,
+            **object
+        )
+
     @appier.ensure(token = "admin.accounts", context = "admin")
     def show_account(self, username):
         account_c = self._get_cls(self.account_c)
@@ -703,40 +814,65 @@ class AdminPart(
             libraries = libraries
         )
 
-    @appier.ensure(context = "admin")
     def oauth_authorize(self):
-        # verifies if the oauth system is allowed if that's not
-        # the case raises an exception indicating so
-        if not self.owner.admin_oauth: raise appier.SecurityError(
-            message = "OAuth not allowed"
-        )
+        try:
+            # verifies if the OAuth system is allowed if that's not
+            # the case raises an exception indicating so
+            if not self.owner.admin_oauth:
+                raise appier.SecurityError(
+                    message = "OAuth not allowed"
+                )
 
-        # retrieves the complete set of fields that are going
-        # to be used on the initial authorize state of oauth
-        client_id = self.field("client_id", mandatory = True)
-        redirect_uri = self.field("redirect_uri", mandatory = True)
-        scope = self.field("scope", mandatory = True)
-        response_type = self.field("response_type", "code")
-        state = self.field("state", None)
+            # retrieves the complete set of fields that are going
+            # to be used on the initial authorize state of OAuth
+            client_id = self.field("client_id", mandatory = True)
+            redirect_uri = self.field("redirect_uri", mandatory = True)
+            scope = self.field("scope", mandatory = True)
+            response_type = self.field("response_type", "code")
+            state = self.field("state", "")
 
-        # verifies/ensures that the response type to be received
-        # is the code, as that's the only one supported
-        appier.verify("response_type", "code")
+            # verifies/ensures that the response type to be received
+            # is the code, as that's the only one supported
+            appier.verify("response_type", "code")
 
-        # normalizes the scope value to the list representation
-        # and then sorts it in the normal manner (normalization)
-        scope_l = scope.split(" ")
-        scope_l.sort()
+            # normalizes the scope value to the list representation
+            # and then sorts it in the normal manner (normalization)
+            scope_l = scope.split(" ")
+            scope_l.sort()
 
-        # runs the mandatory retrieval of the oauth client associated
-        # with the provided client id
-        oauth_client = models.OAuthClient.get_e(
-            client_id = client_id,
-            redirect_uri = redirect_uri
-        )
+            # runs the mandatory retrieval of the OAuth client associated
+            # with the provided client id, notice that the redirect URI is
+            # used as part of the search filter (security measures)
+            oauth_client = models.OAuthClient.get_e(
+                client_id = client_id,
+                redirect_uri = redirect_uri
+            )
 
-        # tries to re-use an already authorized token that is considered
-        # equivalent to the current one, if the re-usage operation is a
+            # asserts that the requested scope is valid for the associated
+            # OAuth client meaning that they overlap
+            oauth_client.assert_scope(scope_l)
+        except Exception as exception:
+            redirect_uri = self.field("redirect_uri", mandatory = True)
+            state = self.field("state", "")
+            if not redirect_uri: raise
+            code = exception.code if hasattr(exception, "code") else None
+            error_message = exception.message if hasattr(exception, "message") else None
+            error_code = OAUTH_ERROR_CODES.get(code, "server_error")
+            return self.redirect(
+                redirect_uri,
+                params = dict(
+                    error = error_code,
+                    error_description = error_message,
+                    state = state
+                )
+            )
+
+        # ensures that that a valid login is provided, this is required
+        # so that it's possible to infer proper authorization
+        appier.ensure_login(self, context = "admin")
+
+        # tries to reuse an already authorized token that is considered
+        # equivalent to the current one, if the reusage operation is a
         # success then redirects the user agent immediately
         result, tokens, oauth_token = models.OAuthToken.reuse_s(
             redirect_uri, scope_l, oauth_client
@@ -746,11 +882,11 @@ class AdminPart(
             params = dict(
                 code = oauth_token.authorization_code,
                 scope = " ".join(oauth_token.tokens),
-                tokens = oauth_token.tokens,
+                state = state
             )
         )
 
-        # runs the template rendering for the oauth authorize panel
+        # runs the template rendering for the OAuth authorize panel
         # it should prompt the final user for permission agreement
         return self.template(
             "oauth/authorize.html.tpl",
@@ -792,7 +928,7 @@ class AdminPart(
             params = dict(
                 code = oauth_token.authorization_code,
                 scope = " ".join(oauth_token.tokens),
-                state = state,
+                state = state
             )
         )
 
@@ -811,7 +947,7 @@ class AdminPart(
         )
 
     def oauth_access_token(self):
-        # verifies if the oauth system is allowed if that's not
+        # verifies if the OAuth system is allowed if that's not
         # the case raises an exception indicating so
         if not self.owner.admin_oauth: raise appier.SecurityError(
             message = "OAuth not allowed"
@@ -823,28 +959,54 @@ class AdminPart(
         client_id = self.field("client_id", mandatory = True)
         client_secret = self.field("client_secret", mandatory = True)
         redirect_uri = self.field("redirect_uri", mandatory = True)
-        code = self.field("code", mandatory = True)
+        code = self.field("code", None)
+        refresh_token = self.field("refresh_token", None)
         grant_type = self.field("grant_type", "authorization_code")
 
-        # tries to retrieve the oauth client associated with the
-        # provided client id and secret and then uses the value to
-        # retrieve the associated oauth token via association
+        # tries to retrieve the OAuth client associated with the
+        # provided client id and secret (client for authentication)
         oauth_client = models.OAuthClient.get_e(
             client_id = client_id,
             client_secret = client_secret,
             redirect_uri = redirect_uri
         )
-        oauth_token = models.OAuthToken.get_e(
-            authorization_code = code,
-            client = oauth_client.id,
-            rules = False
-        )
 
-        # verifies that the authorization code is the expected
-        # one and then unsets it from the oauth token, so that
-        # it's no longer going to be used
-        oauth_token.verify_code(code, grant_type = grant_type)
-        oauth_token.unset_code_s()
+        if grant_type == "authorization_code":
+            # uses the retrieved OAuth client together with the
+            # authorization code to try to retrieve the target token
+            oauth_token = models.OAuthToken.get_e(
+                authorization_code = code,
+                client = oauth_client.id,
+                rules = False
+            )
+
+            # verifies that the authorization code is the expected
+            # one and then unsets it from the OAuth token, so that
+            # it's no longer going to be used
+            oauth_token.verify_code(code, grant_type = grant_type)
+            oauth_token.unset_code_s()
+
+        elif grant_type == "refresh_token":
+            # uses the retrieved OAuth client together with the
+            # refresh token to try to retrieve the target token
+            oauth_token = models.OAuthToken.get_e(
+                refresh_token = refresh_token,
+                client = oauth_client.id,
+                rules = False
+            )
+
+            # verifies that the refresh token contains the expected
+            # value and then if that's the case re-generates a new
+            # access token value to be used
+            oauth_token.verify_refresh(refresh_token, grant_type = grant_type)
+            oauth_token.set_access_token_s()
+
+        else:
+            # raises a security error as no valid grant type was
+            # received (as expected)
+            raise appier.SecurityError(
+                message = "Invalid 'grant_type'"
+            )
 
         # returns the final map based response containing the complete
         # set of elements to the client to be used, notice that both
@@ -866,7 +1028,7 @@ class AdminPart(
             message = "Administration not available"
         )
 
-        # verifies if the oauth system is allowed if that's not
+        # verifies if the OAuth system is allowed if that's not
         # the case raises an exception indicating so
         if not self.owner.admin_oauth: raise appier.SecurityError(
             message = "OAuth not allowed"
@@ -881,7 +1043,7 @@ class AdminPart(
 
         # updates the current session with the proper
         # values to correctly authenticate the user
-        oauth_token._set_session()
+        oauth_token._set_account()
 
         # retrieves the session identifier (SID) for the currently
         # assigned session, this is going to be used in the next
@@ -897,6 +1059,16 @@ class AdminPart(
             tokens = oauth_token.tokens
         )
 
+    @appier.ensure(token = "admin", context = "admin")
+    def restart_app(self):
+        self.owner.trigger_bus("restart")
+        return self.redirect(
+            self.url_for(
+                "admin.operations",
+                message = "Application restart triggered"
+            )
+        )
+
     @appier.ensure(token = "admin", context = "admin")
     def build_index(self):
         empty = self.field("empty", True, cast = bool)
@@ -934,12 +1106,20 @@ class AdminPart(
         receiver = appier.conf("TEST_EMAIL", None)
         receiver = self.field("email", receiver)
         receiver = self.field("receiver", receiver)
+        template = self.field("template", "test")
+        render = self.field("render", False, cast = bool)
+        if render: return self.template(
+            "admin/email/%s.html.tpl" % template,
+            layout = False,
+            subject = "Test email",
+            title = "Test email"
+        )
         if not receiver: raise appier.OperationalError(
             message = "No test email defined"
         )
         models.Base.send_email_g(
             self.owner,
-            "admin/email/test.html.tpl",
+            "admin/email/%s.html.tpl" % template,
             receivers = [receiver],
             subject = "Test email",
             title = "Test email"
@@ -1021,6 +1201,16 @@ class AdminPart(
             )
         )
 
+    @appier.ensure(token = "admin.status", context = "admin")
+    def list_peers(self):
+        peers = appier.legacy.items(self.owner._peers)
+        peers.sort()
+        return self.template(
+            "peers.html.tpl",
+            section = "status",
+            peers = peers
+        )
+
     @appier.ensure(token = "admin.status", context = "admin")
     def list_counters(self):
         collection = self._counters()
@@ -1057,6 +1247,20 @@ class AdminPart(
         self.content_type("text/csv")
         return result
 
+    @appier.ensure(token = "admin", context = "admin")
+    def bundle_locale_json(self, id):
+        name = self.field("name", None)
+        locale = models.Locale.get(id = id, rules = False)
+        name = name or locale.context or "global"
+        file_name = "%s.%s.json" % (name, locale.locale)
+        self.content_disposition("filename=\"%s\"" % file_name)
+        return self.json(
+            locale.data_j,
+            sort_keys = True,
+            indent = 4,
+            separators = (",", ": ")
+        )
+
     @appier.ensure(token = "admin.database", context = "admin")
     def database(self):
         return self.template(
@@ -1079,10 +1283,7 @@ class AdminPart(
         file_name = "%s_%s.dat" % (self.owner.name, date_time_s)
 
         self.content_type("application/octet-stream")
-        self.request.set_header(
-            "Content-Disposition",
-            "attachment; filename=\"%s\"" % file_name
-        )
+        self.content_disposition("attachment; filename=\"%s\"" % file_name)
 
         return file.getvalue()
 
@@ -1585,7 +1786,13 @@ class AdminPart(
         )
         code = self.field("code")
         state = self.field("state")
-        context, next = state.split(":", 1)
+        error = self.field("error")
+        appier.verify(
+            not error,
+            message = "Invalid OAuth response (%s)" % error,
+            exception = appier.OperationalError
+        )
+        context, next = state.split(":", 1) if state else (state, None)
         api = self.get_facebook_api()
         access_token = api.oauth_access(code)
         if context == "login":
@@ -1642,7 +1849,13 @@ class AdminPart(
         )
         code = self.field("code")
         state = self.field("state")
-        context, next = state.split(":", 1)
+        error = self.field("error")
+        appier.verify(
+            not error,
+            message = "Invalid OAuth response (%s)" % error,
+            exception = appier.OperationalError
+        )
+        context, next = state.split(":", 1) if state else (state, None)
         api = self.get_github_api()
         access_token = api.oauth_access(code)
         if context == "login":
@@ -1706,7 +1919,13 @@ class AdminPart(
         )
         code = self.field("code")
         state = self.field("state")
-        context, next = state.split(":", 1)
+        error = self.field("error")
+        appier.verify(
+            not error,
+            message = "Invalid OAuth response (%s)" % error,
+            exception = appier.OperationalError
+        )
+        context, next = state.split(":", 1) if state else (state, None)
         api = self.get_google_api()
         access_token = api.oauth_access(code)
         if context == "login":
@@ -1771,7 +1990,13 @@ class AdminPart(
         )
         code = self.field("code")
         state = self.field("state")
-        context, next = state.split(":", 1)
+        error = self.field("error")
+        appier.verify(
+            not error,
+            message = "Invalid OAuth response (%s)" % error,
+            exception = appier.OperationalError
+        )
+        context, next = state.split(":", 1) if state else (state, None)
         api = self.get_live_api()
         access_token = api.oauth_access(code)
         if context == "login":
@@ -1825,8 +2050,13 @@ class AdminPart(
             message = "Administration not available"
         )
         oauth_verifier = self.field("oauth_verifier")
+        appier.verify(
+            oauth_verifier,
+            message = "Invalid OAuth response",
+            exception = appier.OperationalError
+        )
         state = self.field("state")
-        context, next = state.split(":", 1)
+        context, next = state.split(":", 1) if state else (state, None)
         api = self.get_twitter_api()
         oauth_token, oauth_token_secret = api.oauth_access(oauth_verifier)
         if context == "login":
@@ -1881,7 +2111,7 @@ class AdminPart(
 
         # updates the current session with the proper
         # values to correctly authenticate the user
-        account._set_session()
+        account._set_account()
 
         # retrieves the session identifier (SID) for the currently
         # assigned session, this is going to be used in the next
