@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -39,7 +39,7 @@ __license__ = "Apache License, Version 2.0"
 
 import appier
 
-class Authenticable(object):
+class Authenticable(appier.Observable):
 
     @classmethod
     def _unset_account(cls, prefixes = None, safes = [], method = "delete"):
@@ -47,10 +47,27 @@ class Authenticable(object):
         _cls = session.get("cls", None)
         if _cls: cls = appier.get_model(_cls)
         cls._unset_session(prefixes = prefixes, safes = safes, method = method)
+        Authenticable.trigger_g(
+            "unset_account",
+            prefixes = prefixes,
+            safes = safes,
+            method = method
+        )
 
     @classmethod
     def _unset_session(cls, prefixes = None, safes = [], method = "delete"):
         pass
 
+    def _set_account(self, unset = True, safes = [], method = "set"):
+        cls = self.__class__
+        self._set_session(unset = unset, safes = safes, method = method)
+        Authenticable.trigger_g(
+            "set_account",
+            self,
+            unset = unset,
+            safes = safes,
+            method = method
+        )
+
     def _set_session(self, unset = True, safes = [], method = "set"):
         pass
