@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2018 Hive Solutions Lda.
+# Copyright (c) 2008-2019 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2018 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -44,6 +44,9 @@ import xml.sax.saxutils
 
 import appier
 
+try: import regex
+except ImportError: regex = None
+
 class MarkdownParser(object):
     """
     Parser object for the md (Markdown) format, should be able to
@@ -63,18 +66,26 @@ class MarkdownParser(object):
     def __init__(self):
         newline = r"(?P<newline>(\n\n)|(\r\n\r\n))"
         header = r"(?P<header>^(?P<header_index>#+) (?P<header_value>.+)$)"
-        list = r"(?P<list>^(?P<list_index>[ \t]*)[\*\+\-] (?P<list_value>[^\r\n]+))"
-        listo = r"(?P<listo>^(?P<listo_index>[ \t]*)(?P<listo_number>\d+)\. (?P<listo_value>[^\r\n]+))"
+        list = r"(?P<list>^(?P<list_index>[ \t]*)[\*\+\-](?:(?:[ \t]+(?P<list_value>[^\r\n]*))|(?:[ \t]*$)))"
+        listo = r"(?P<listo>^(?P<listo_index>[ \t]*)(?P<listo_number>\d+)\.(?:(?:[ \t]+(?P<listo_value>[^\r\n]*))|(?:[ \t]*$)))"
         blockquote = r"(?P<blockquote>^[\>][ \t]*(?P<blockquote_value>[^\r\n]*))"
-        image = r"(?P<image>\!(?P<image_label>\[.+?\])(?P<image_value>\(.+?\)))"
-        link = r"(?P<link>(?P<link_label>\[.+?\])(?P<link_value>\([^ !]+\)))"
+        image = r"(?P<image>\!(?P<image_label>\[.+\])(?P<image_value>\(.+?\)))"
+        link = r"(?P<link>(?P<link_label>\[(?:(?:\!.+)|(?:[^\]]+))\])(?P<link_value>\(.+?\)))"
         bold = r"(?P<bold>\*\*(?P<bold_value>[^\0]+?)\*\*)"
         italic = r"(?P<italic>\*(?P<italic_value>[^\0]+?)\*)"
+        table_header = r"(?P<table_header>(?P<table_header_value>\|[ \-]{3,999})+\|)"
+        table_line = r"(?P<table_line>(?P<table_line_value>\|[^\n]*)\|)"
         code = r"(?P<code>```(?P<code_name>.*)(?P<code_value>[^\0]+?)```)"
-        code_line = r"(?P<code_line>^(    |\t)(?P<code_line_value>[^\r\n]+))"
+        code_line = r"(?P<code_line>^(?:    |\t)(?P<code_line_value>[^\r\n]+))"
         code_single = r"(?P<code_single>`?`(?P<code_single_value>[^`]+)``?)"
 
-        self.master = re.compile(
+        if regex:
+            image = r"(?P<image>\!(?P<image_label>\[(?>[^\[\]]|(?P>image_label))+\])(?P<image_value>\((?>[^()]|(?P>image_value))+\)))"
+            link = r"(?P<link>(?P<link_label>\[(?>[^\[\]]|(?P>link_label))+\])(?P<link_value>\((?>[^()]|(?P>link_value))+\)))"
+
+        rex = regex or re
+
+        self.master = rex.compile(
             "|".join([
                 newline,
                 header,
@@ -85,13 +96,15 @@ class MarkdownParser(object):
                 link,
                 bold,
                 italic,
+                table_header,
+                table_line,
                 code,
                 code_line,
                 code_single
             ]),
-            re.MULTILINE | re.UNICODE
+            rex.MULTILINE | rex.UNICODE
         )
-        self.simple = re.compile(
+        self.simple = rex.compile(
             "|".join([
                 image,
                 link,
@@ -99,7 +112,7 @@ class MarkdownParser(object):
                 italic,
                 code_single
             ]),
-            re.MULTILINE | re.UNICODE
+            rex.MULTILINE | rex.UNICODE
         )
         self.ids = []
 
@@ -160,7 +173,8 @@ class MarkdownParser(object):
 
     def parse_list(self, parts):
         index = parts["list_index"]
-        value = parts["list_value"]
+        value = parts["list_value"] or ""
+        value = value.strip()
         value = self.parse(value, regex = self.simple)
 
         node = dict(
@@ -173,7 +187,7 @@ class MarkdownParser(object):
     def parse_listo(self, parts):
         index = parts["listo_index"]
         number = parts["listo_number"]
-        value = parts["listo_value"]
+        value = parts["listo_value"] or ""
         value = self.parse(value, regex = self.simple)
 
         node = dict(
@@ -212,12 +226,8 @@ class MarkdownParser(object):
         label = parts["link_label"]
         value = parts["link_value"]
 
-        original = label + value
-        reversed = original[::-1]
-        last = reversed.index("(")
-
-        label = original[1:(last + 2) * -1]
-        value = original[last * -1:-1]
+        label = label[1:-1]
+        value = value[1:-1]
 
         label = self.parse(label, regex = self.simple)
 
@@ -248,6 +258,29 @@ class MarkdownParser(object):
         )
         return node
 
+    def parse_table_header(self, parts):
+        value = parts["table_header_value"]
+        value = value.strip("|")
+        values = [item.strip() for item in value.split("|")]
+
+        node = dict(
+            type = "table_header",
+            values = values,
+        )
+        return node
+
+    def parse_table_line(self, parts):
+        value = parts["table_line_value"]
+        value = value.strip("|")
+        values = [item.strip() for item in value.split("|")]
+        values = [self.parse(value, regex = self.simple) for value in values]
+
+        node = dict(
+            type = "table_line",
+            values = values,
+        )
+        return node
+
     def parse_code(self, parts):
         name = parts["code_name"]
         value = parts["code_value"]
@@ -375,6 +408,73 @@ class MarkdownGenerator(object):
             method = getattr(self, "generate_" + _type)
             method(node)
 
+class MarkdownDebug(MarkdownGenerator):
+
+    def __init__(
+        self,
+        file = None,
+        options = dict(extended = False),
+        encoding = "utf-8"
+    ):
+        MarkdownGenerator.__init__(
+            self,
+            file = file,
+            encoding = encoding,
+            options = options
+        )
+        self.first = True
+        self.extended = self.options.get("extended", False)
+
+    def emit(self, value):
+        value = ("" if self.first else ",") + value
+        self.first = False
+        MarkdownGenerator.emit(self, value)
+
+    def reset(self):
+        MarkdownGenerator.reset(self)
+        self.first = True
+
+    def generate_newline(self, node):
+        self.emit("newline")
+
+    def generate_header(self, node):
+        self.emit("header")
+
+    def generate_list(self, node):
+        self.emit("list")
+
+    def generate_listo(self, node):
+        self.emit("listo")
+
+    def generate_image(self, node):
+        if self.extended: self.emit("image(value=%s)" % node["value"])
+        else: self.emit("image")
+
+    def generate_link(self, node):
+        if self.extended: self.emit("link(value=%s)" % node["value"])
+        else: self.emit("link")
+
+    def generate_bold(self, node):
+        self.emit("bold")
+
+    def generate_italic(self, node):
+        self.emit("italic")
+
+    def generate_table_header(self, node):
+        self.emit("table_header")
+
+    def generate_table_line(self, node):
+        self.emit("table_line")
+
+    def generate_code(self, node):
+        self.emit("code")
+
+    def generate_blockquote(self, node):
+        self.emit("blockquote")
+
+    def generate_normal(self, node):
+        self.emit("normal")
+
 class MarkdownHTML(MarkdownGenerator):
 
     def __init__(
@@ -415,6 +515,8 @@ class MarkdownHTML(MarkdownGenerator):
         MarkdownGenerator.reset(self)
         self.depth = 0
         self.paragraph = False
+        self.table = False
+        self.table_head = False
         self.code = False
         self.blockquote = False
         self.list_item = False
@@ -488,6 +590,23 @@ class MarkdownHTML(MarkdownGenerator):
         self._generate(value)
         self.close("</em>")
 
+    def generate_table_header(self, node):
+        if not self.table_head: return
+        self.close("</thead>")
+        self.open("<tbody>")
+        self.table_head = False
+
+    def generate_table_line(self, node):
+        values = node["values"]
+        self._ensure_table()
+        tag = "th" if self.table_head else "td"
+        self.emit("<tr>")
+        for value in values:
+            self.emit("<%s>" % tag)
+            self._generate(value)
+            self.emit("</%s>" % tag)
+        self.emit("</tr>")
+
     def generate_code(self, node):
         value = node["value"]
         name = node.get("name", "undefined")
@@ -511,6 +630,13 @@ class MarkdownHTML(MarkdownGenerator):
     def is_absolute(self, url):
         return url.startswith(("http://", "https://"))
 
+    def _ensure_table(self):
+        if self.table: return
+        self.open("<table>")
+        self.open("<thead>")
+        self.table = True
+        self.table_head = True
+
     def _ensure_code(self, tag, name):
         if self.code: return
         self.open("<%s class=\"code language-%s\">" % (tag, name))
@@ -541,6 +667,7 @@ class MarkdownHTML(MarkdownGenerator):
         if not "listo" in exceptions: self._close_listo()
         if not "list" in exceptions: self._close_list()
         if not "blockquote" in exceptions: self._close_blockquote()
+        if not "table" in exceptions: self._close_table()
         if not "code" in exceptions: self._close_code()
         if not "paragraph" in exceptions: self._close_paragraph()
 
@@ -549,6 +676,14 @@ class MarkdownHTML(MarkdownGenerator):
         self.close("</p>")
         self.paragraph = False
 
+    def _close_table(self):
+        if not self.table: return
+        if self.table_head: self.close("</thead>")
+        else: self.close("</tbody>")
+        self.close("</table>")
+        self.table = False
+        self.table_head = False
+
     def _close_code(self, tag = "pre"):
         if not self.code: return
         self.close("</%s>" % tag)
@@ -578,6 +713,9 @@ class MarkdownHTML(MarkdownGenerator):
         escaped = xml.sax.saxutils.escape(value_s)
         return escaped if appier.legacy.PYTHON_3 else escaped.decode(encoding)
 
+def has_regex():
+    return True if regex else False
+
 def run():
     if len(sys.argv) < 3:
         print("Invalid number of arguments")
